//
//  VideoEditor.swift
//  EnvironmentSetup
//
//  Created by Aliaksandr Andrashuk on 01/07/2023.
//

import Foundation
import AVFoundation
import CoreImage
import CoreImage.CIFilterBuiltins

class FilterVideoComposition {
    var avPlayerItem: AVPlayerItem? = nil
    var avPlayer: AVPlayer? = nil

    private var asset: AVAsset

    init(url: URL) async {
        let url = url
        let asset = AVAsset(url: url)
        let item = AVPlayerItem(asset: asset)

        self.asset = asset
        self.avPlayerItem = item
        self.avPlayer = AVPlayer(playerItem: item)

        await buildVideo()
    }

    func buildVideo() async {
        guard let item = avPlayerItem else {return}

        item.videoComposition = try? await buildComposition()
    }

    private func buildComposition() async throws -> AVVideoComposition {
        let maxScale = 200.0
        let filter = CIFilter.hexagonalPixellate()
        let duration = try await asset.load(.duration)
        let composition = AVMutableVideoComposition(asset: asset) { request in
            let sourceImage = request.sourceImage
            let time = request.compositionTime
            let ratio = time.seconds / duration.seconds

            filter.scale = Float(max(1.0, maxScale * ratio))
            filter.inputImage = sourceImage

            let outputImage = filter.outputImage ?? sourceImage

            request.finish(with: outputImage, context: nil)
        }

        return composition
    }
}
